pro xphot, distance_pc

  ;;; This is an implementation of our non-parametric method for estimation of apparent and intrinsic
  ;;; broad-band fluxes and absorbing X-ray column densities of weak X-ray sources. The method is
  ;;; described in the "Methods for Estimating Fluxes and Absorptions of Faint X-ray Sources" by
  ;;; Getman, Feigelson, Broos, Townsley, Garmire (2010; http://adsabs.harvard.edu/abs/2010ApJ...708.1760G)
  ;;; paper:
  ;;; \bibitem[Getman et al.(2010)]{2010ApJ...708.1760G} Getman, K.~V., Feigelson, E.~D., Broos, P.~S.,
  ;;;  Townsley, L.~K., \& Garmire, G.~P.\ 2010, \apj, 708, 1760 
  ;;;
  ;;; Absorption and intrinsic flux are estimated from the comparison of the apparent median 
  ;;; energy of the source photons and apparent source flux with those of high signal-to-noise
  ;;; spectra that were simulated using spectral models characteristic of much brighter sources
  ;;; of similar class of sources previously studied in detail.
  ;;;
  ;;; The results of the simulations are expected to be given in the form of 4 ASCII tables (following 
  ;;; the Getman et al. paper's table numbering convention): table 2 (flux bias versus median 
  ;;; energy), table 3 (simulated column density versus median energy), table 4 (simulated intrinsic
  ;;; to apparent flux ratio versus median energy), and table 5 (simulated errors on median energy
  ;;; versus median energy). As an example, the 4 calibration tables for the case of the Chandra X-ray data
  ;;; from pre-main sequence (PMS) stars are given in the paper and provided with this code.
  ;;; Researchers working with the data from other X-ray telescopes or source classes are advised to
  ;;; produce their own calibration tables.
  ;;;
  ;;; The algorithm of the method is presented in the figures fig_flow_chart_part1.pdf and
  ;;; fig_flow_chart_part2.pdf.
  ;;;
  ;;; The inputs for this IDL program (xphot.pro) are: 
  ;;; 1) distance to studied source(s) in parsecs;
  ;;; 2) calibration data from the 4 ASCII calibration tables;
  ;;; 3) measured photometric (apparent) source properties in the full and hard energy bands, such as
  ;;; source's median energies, photometric fluxes, net counts, and errors on net counts.

  ;;; Distance should be given as a command line input parameter. Calibration data are read from the
  ;;; four ASCII tables (STEP 0). Apparent source properties are read from the collated fits file 
  ;;; that is a standard product of the Acis Extract program (ascl.net/1203.001; STEP 1). As an example, 
  ;;; the collated fits files for the young stars in the M17 and W40 star forming regions are provided
  ;;; with this code (m17_collated.fits and w40_collated.fits).

  ;;; Apparent and intrinsic fluxes and absorbing X-ray column densities are derived in STEPS 2-4, and their
  ;;; errors --- in STEPS 5-6. These derived quantities are retained in the IDL sav file, xphot_out.sav (STEP 7).

  ;;; Code: xphot.pro ; Version: 09/09/09 ; Author: K. V. Getman (gkosta@astro.psu.edu)
 

 
  distance_pc = double(distance_pc)
  print, "Distance (pc):",distance_pc

  ;; STEP 0. Read 4 calibration tables.
  print, "STEP 0. Reading calibration tables."
  readcol, 'table2.txt', me3, flux_bias3, model3, band3, F='D,D,D,A' ; table 2
  readcol, 'table3.txt', me4, nh4, model4, F='D,D,D'                 ; table 3
  readcol, 'table4.txt', me5, flux_ratio5, model5, band5, F='D,D,D,A' ; table 4
  readcol, 'table5.txt', me6, mede_err6, model6, band6, nc6, F='D,D,D,A,I' ; table 5
  

  ;; STEP 1. Read AE collated file to get Fh_phot, Ft_phot, MedEh, MedEt, NCh, NCt, sNCh, sNCt.
  print, "STEP 1. Reading AE photometry information."
  bt = mrdfits('m17_collated.fits', 1)
  band_full = 0 
  print, 'As a full-band using the energy band ', bt[0].ENERG_LO[band_full], bt[0].ENERG_HI[band_full]
  band_hard = 2
  print, 'As a hard-band using the energy band ', bt[0].ENERG_LO[band_hard], bt[0].ENERG_HI[band_hard]

  Fh_phot_out = bt.flux2[band_hard]
  Ft_phot_out = bt.flux2[band_full]
  MedEh_out = bt.ENERG_PCT50_OBSERVED[band_hard]
  MedEt_out = bt.ENERG_PCT50_OBSERVED[band_full]
  NCh_out = bt.NET_CNTS[band_hard]
  NCt_out = bt.NET_CNTS[band_full]
  sNCh_out = (bt.NET_CNTS_SIGMA_UP[band_hard] + bt.NET_CNTS_SIGMA_LOW[band_hard])/2.
  sNCt_out = (bt.NET_CNTS_SIGMA_UP[band_full] + bt.NET_CNTS_SIGMA_LOW[band_full])/2.

  ;; Apply simple criterion to skip very weak sources for which even the photometry
  ;; method fails. Use your own criteria here. I use NCt>=5 and NCh >=3 counts.
  ind_filter = where((NCt_out ge 5) and (NCh_out ge 3),ccc_filter, complement=ind_left)
  print, "The method will be applied to ",ccc_filter," sources."
  print, "The method will not be applied to the remaining ",n_elements(ind_left)," sources."
  ccc_all = ccc_filter + n_elements(ind_left)
  Fh_phot  = Fh_phot_out[ind_filter]
  Ft_phot  = Ft_phot_out[ind_filter]
  MedEh = MedEh_out[ind_filter]
  MedEt = MedEt_out[ind_filter]
  NCh = NCh_out[ind_filter]
  NCt = NCt_out[ind_filter]
  sNCh = sNCh_out[ind_filter]
  sNCt = sNCt_out[ind_filter]


     print, "STEPS 2-4. Deriving apparent and intrinsic fluxes and absorbing column densities."  
  ;; STEP 2. Obtain best model family (bmf).
     
   ; STEP 2.1. Pass Fh_phot (photon/s/cm2) and MedEh (keV) through the Equation 4 to get Fh (erg/s/cm2). This is still a "biased" Fh.
   Fh =  1.602d-9 * Fh_phot * MedEh ; is linear version; this is a "biased" version of Fh, sufficient to get bmf; later we will get a de-biased version of Fh.


   ; STEPS 2.2 and 4.1. Pass Fh, MedEt, and Distance through the calibration data of Table 4 to obtain the "best model family" (bmf) and log(Fhc/Fh)sim flux ratio.
   ; If a source is not heavily obscured (MedEt <2 keV), then the "best model family" is obtained directly from source's Lh value.
   ; If a source is heavily obscured (MedEt >= 2 keV), then the de-reddening procedure is applied to get the "best model family".
   bmf_template = [27.0, 27.5, 28.0, 28.5, 29.0, 29.5, 30.0, 30.5, 31.0, 31.5, 32.0]
   Lh = alog10(Fh * 4.*!PI*(3.08568d18*Distance_pc)^2) ; is Log version
   
   bmf=fltarr(ccc_filter) ; is Log version
   flux_ratio_hard=dblarr(ccc_filter) ; is Log version
   for i = 0,ccc_filter-1 do begin ; loop over all sources to which the method is applied
     if (MedEt[i] lt 2.) then begin
       ; Get bmf of an un-obscured source.
       ind_close = where(abs(Lh[i] - bmf_template) eq min(abs(Lh[i] - bmf_template)),ccc_close) & ind_close0 = ind_close[0]
       bmf[i] = bmf_template[ind_close0]
       ; get log(Fhc/Fh)sim from Figure 5 (me5, flux_ratio5, model5, band5)
       ind_subset = where((band5 eq 'hard') and (model5 eq bmf[i]),ccc_subset)
       me5_subset = me5[ind_subset] & flux_ratio5_subset = flux_ratio5[ind_subset]
       ind_close = where(abs(MedEt[i] - me5_subset) eq min(abs(MedEt[i] - me5_subset)),ccc_close) & ind_close0 = ind_close[0]
       flux_ratio_hard[i] = flux_ratio5_subset[ind_close0]
     endif else begin
       ; Get bmf of an obscured source.
       ; Based on MedEt and Figure 5b get 11 possible flux ratios (remind that flux ratios are in Log).
       ; Of those 11, the best is the one with its Lhc = flux_ratio + Lh to be the closest to the "nominal" COUP model Lh!
         temp_delta_lhc = 10000.
         for j = 0,n_elements(bmf_template)-1 do begin ; loop over 11 model families
           ind_subset = where((band5 eq 'hard') and (model5 eq bmf_template[j]),ccc_subset)
           me5_subset = me5[ind_subset] & flux_ratio5_subset = flux_ratio5[ind_subset]
           ind_close = where(abs(MedEt[i] - me5_subset) eq min(abs(MedEt[i] - me5_subset)),ccc_close) & ind_close0 = ind_close[0]
           
           temp_flux_ratio_hard = flux_ratio5_subset[ind_close0]
           EXPECTED_LHC = temp_flux_ratio_hard + Lh[i]
           DELTA_LHC = abs(EXPECTED_LHC - bmf_template[j]) 

           if (DELTA_LHC lt temp_delta_lhc) then begin          
             flux_ratio_hard[i] = temp_flux_ratio_hard
             temp_delta_lhc = DELTA_LHC
             bmf[i] = bmf_template[j]
           endif

         endfor ; end loop over 11 model families
       
     endelse
   endfor ; end loop over all sources to which the method is applied

  
   ; STEP 4.2. Pass MedEt and "best model family" through the calibration data of Table 4 to obtain the log(Ftc/Ft)sim flux ratio.
   flux_ratio_full = dblarr(ccc_filter) ; is Log version
   for i = 0,ccc_filter-1 do begin ; loop over all sources to which the method is applied
     ; Get log(Ftc/Ft)sim from Table 4 (me5, flux_ratio5, model5, band5)
       ind_subset = where((band5 eq 'full') and (model5 eq bmf[i]),ccc_subset)
       me5_subset = me5[ind_subset] & flux_ratio5_subset = flux_ratio5[ind_subset]
       ind_close = where(abs(MedEt[i] - me5_subset) eq min(abs(MedEt[i] - me5_subset)),ccc_close) & ind_close0 = ind_close[0]
       flux_ratio_full[i] = flux_ratio5_subset[ind_close0] 
   endfor ; end loop over all sources to which the method is applied


   ; STEPS 3.1 and 4.3. Pass MedEt,MedEh, Fh_phot, log(Fhc/Fh)sim, and "best model family" (bmf) through the Equation 4 and Table 2 to obtain Fh and Fhc
   Fh = dblarr(ccc_filter) ; in erg/cm2/s, linear version
   Fhc = dblarr(ccc_filter) ; in erg/cm2/s, linear version 
   for i = 0,ccc_filter-1 do begin ; loop over all sources to which the method is applied
     ; Get flux bias (linear) from Table 2 (me3, flux_bias3, model3, band3)
       ind_subset = where((band3 eq 'hard') and (model3 eq bmf[i]),ccc_subset)
       me3_subset = me3[ind_subset] & flux_bias3_subset = flux_bias3[ind_subset]
       ind_close = where(abs(MedEt[i] - me3_subset) eq min(abs(MedEt[i] - me3_subset)),ccc_close) & ind_close0 = ind_close[0]
       temp_flux_bias = flux_bias3_subset[ind_close0]
       ; De-bias AE photometry flux which is btw in phot/cm2/s
       temp_Fh_phot = Fh_phot[i]/temp_flux_bias
       ; Get apparent flux in ergs/cm2/s
       Fh[i] =  1.602d-9 * temp_Fh_phot * MedEh[i]
       ; Get corrected for absorption flux in ergs/cm2/s units
       Fhc[i] = 10.^flux_ratio_hard[i] * Fh[i]       
   endfor ; end loop over all sources to which the method is applied



   ; STEPS 3.2 and 4.4. Pass MedEt, Ft_phot, log(Ftc/Ft)sim, and "best model family" (bmf) through the Equation 3 and Table 2 to obtain Ft and Ftc
   Ft = dblarr(ccc_filter) ; in erg/cm2/s, linear version
   Ftc = dblarr(ccc_filter) ; in erg/cm2/s, linear version 
   for i = 0,ccc_filter-1 do begin ; loop over all sources to which the method is applied
     ; Get flux bias (linear) from Table 2 (me3, flux_bias3, model3, band3)
       ind_subset = where((band3 eq 'full') and (model3 eq bmf[i]),ccc_subset)
       me3_subset = me3[ind_subset] & flux_bias3_subset = flux_bias3[ind_subset]
       ind_close = where(abs(MedEt[i] - me3_subset) eq min(abs(MedEt[i] - me3_subset)),ccc_close) & ind_close0 = ind_close[0]
       temp_flux_bias = flux_bias3_subset[ind_close0]
       ; De-bias AE photometry flux which is btw in phot/cm2/s
       temp_Ft_phot = Ft_phot[i]/temp_flux_bias
       ; Get apparent flux in ergs/cm2/s
       Ft[i] =  1.602d-9 * temp_Ft_phot * MedEt[i]
       ; Get corrected for absorption flux in ergs/cm2/s units
       Ftc[i] = 10.^flux_ratio_full[i] * Ft[i]       
   endfor ; end loop over all sources to which the method is applied


   ; STEP 3.3. Pass MedEt and "best model family" (bmf) through the Table 3 to obtain logNH
   logNH = dblarr(ccc_filter) ; in cm-2, Log version of absorbing column density
   for i = 0,ccc_filter-1 do begin ; loop over all sources to which the method is applied
     ; Get NH from Table 3 (me4, nh4, model4)
     ind_subset = where((model4 eq bmf[i]),ccc_subset)
     me4_subset = me4[ind_subset] & nh4_subset = nh4[ind_subset]
     ind_close = where(abs(MedEt[i] - me4_subset) eq min(abs(MedEt[i] - me4_subset)),ccc_close) & ind_close0 = ind_close[0]
     logNH[i] = nh4_subset[ind_close0]  
   endfor ; end loop over all sources to which the method is applied



   ;;========================================================================================================================
   print, "STEPS 5-6. Deriving errors on apparent and intrinsic fluxes and absorbing column densities."
   ;; STEPS 5-6. Get errors on Fh, Fhc, Ft, Ftc, logNH
        
   ; STEP 5.1. Pass MedEt, MedEh, NCt, NCh through the calibration data of Table 5 to get statistical errors on MedE (sMedEt and sMedEh)
   sMedEt = dblarr(ccc_filter)
   sMedEh = dblarr(ccc_filter)
   ; Notice that in Figure 6 the NC strata are different between the 'hard' and 'full' bands:
   nc6_strata_hard = [5, 10, 20, 40, 60, 100, 200]
   nc6_strata_full = [5, 10, 20, 40, 60, 100, 500, 1000]
   for i = 0,ccc_filter-1 do begin ; loop over all sources to which the method is applied
      ; Get errors on MedE from Table 5 (me6, mede_err6, model6, band6, nc6)
      ; Hard band
      ind_close=where(abs(NCh[i]-nc6_strata_hard) eq min(abs(NCh[i]-nc6_strata_hard)),ccc_close) & ind_close0 = ind_close[0]
      nc6_subset = nc6_strata_hard[ind_close0]
      ind_subset = where((model6 eq bmf[i]) and (band6 eq 'hard') and (nc6 eq nc6_subset),ccc_subset)
      me6_subset = me6[ind_subset] & mede_err6_subset = mede_err6[ind_subset]
      ind_close = where(abs(MedEh[i] - me6_subset) eq min(abs(MedEh[i] - me6_subset)),ccc_close) & ind_close0 = ind_close[0]
      sMedEh[i] = mede_err6_subset[ind_close0]


      ; Full band
      ind_close=where(abs(NCt[i]-nc6_strata_full) eq min(abs(NCt[i]-nc6_strata_full)),ccc_close) & ind_close0 = ind_close[0]
      nc6_subset = nc6_strata_full[ind_close0]
      ind_subset = where((model6 eq bmf[i]) and (band6 eq 'full') and (nc6 eq nc6_subset),ccc_subset)
      me6_subset = me6[ind_subset] & mede_err6_subset = mede_err6[ind_subset]
      ind_close = where(abs(MedEt[i] - me6_subset) eq min(abs(MedEt[i] - me6_subset)),ccc_close) & ind_close0 = ind_close[0]
      sMedEt[i] = mede_err6_subset[ind_close0]
   endfor ; end loop over all sources to which the method is applied 


   ; STEP 5.2. Pass sNCt, sNCh, sMedEt, sMedEh, Ft, Fh, NCt, NCh, MedEt, MedEh through Equations 1-4 to get statistical errors on apparent fluxes (sFt and sFh)
   ; Use error propagation from e.g. Bevington & Robinson (1992):
   ; Y = a * X1 * X2, where "a" is a constant, and X1 and X2 are independent variables. Error propagation is sY^2 = a^2 * (X2^2 * sX1^2 + X1^2 * sX2^2)
   sFh = sqrt((Fh/NCh*sNCh)^2 + (Fh/MedEh*sMedEh)^2)
   sFt = sqrt((Ft/NCt*sNCt)^2 + (Ft/MedEt*sMedEt)^2)


   ; STEP 6.1. Pass sMedEt, MedEt, logNH, bmf through the Table 3 to obtain statistical and systematic errors on logNH (slogNH_stat, slogNH_syst)
   slogNH_stat = dblarr(ccc_filter)
   slogNH_syst = dblarr(ccc_filter)
   for i = 0,ccc_filter-1 do begin ; loop over all sources to which the method is applied
      ; Get statistical errors on logNH using Table 3 (me4, nh4, model4)
      ind_subset = where((model4 eq bmf[i]),ccc_subset)
      me4_subset = me4[ind_subset] & nh4_subset = nh4[ind_subset]
      MedEt_lo = MedEt[i] - sMedEt[i]
      MedEt_hi = MedEt[i] + sMedEt[i] 
      ind_close_lo = where(abs(MedEt_lo - me4_subset) eq min(abs(MedEt_lo - me4_subset)),ccc_close_lo) & ind_close_lo0 = ind_close_lo[0]
      ind_close_hi = where(abs(MedEt_hi - me4_subset) eq min(abs(MedEt_hi - me4_subset)),ccc_close_hi) & ind_close_hi0 = ind_close_hi[0]
      slogNH_stat[i] = abs(nh4_subset[ind_close_hi0]-nh4_subset[ind_close_lo0])/2.
  
      ; Get systematic errors on logNH using Table 3 (me4, nh4, model4)
      ; Get the "second nearest neighbour" model curves (call them left_model and right_model). 
      ; In the cases where bmf =[27, 27.5, 31.5, 32], we will use one-sided errors, or rather maximum value among low and hi systematic errors. 
      if (bmf[i] gt 27.5) then begin
         left_model = bmf[i] - 1
      endif else begin
         left_model = 27.0
      endelse

      if (bmf[i] lt 31.5) then begin
         right_model = bmf[i] + 1 
      endif else begin
         right_model = 32.0
      endelse
      
      ind_subset_left = where((model4 eq left_model),ccc_subset_left)
      me4_subset_left = me4[ind_subset_left] & nh4_subset_left = nh4[ind_subset_left]
      ind_close_left = where(abs(MedEt[i] - me4_subset_left) eq min(abs(MedEt[i] - me4_subset_left)),ccc_close_left) & ind_close_left0 = ind_close_left[0]

      ind_subset_right = where((model4 eq right_model),ccc_subset_right)
      me4_subset_right = me4[ind_subset_right] & nh4_subset_right = nh4[ind_subset_right]
      ind_close_right = where(abs(MedEt[i] - me4_subset_right) eq min(abs(MedEt[i] - me4_subset_right)),ccc_close_right) & ind_close_right0 = ind_close_right[0]

      syst_arr = [abs(logNH[i] - nh4_subset_left[ind_close_left0]), abs(logNH[i] - nh4_subset_right[ind_close_right0])]
      slogNH_syst[i] = max(syst_arr)
   endfor ; end loop over all sources to which the method is applied


   
   ; STEP 6.2. Pass sMedEt, MedEt, sFt, sFh, Ft, Fh, Ftc, Fhc, bmf through Table 4 to obtain statistical and systematic errors on Ftc and Fhc (sFtc_stat, sFtc_syst, sFhc_stat, sFhc_syst) 
    sFtc_stat = dblarr(ccc_filter)
    sFtc_syst = dblarr(ccc_filter)
    sFhc_stat = dblarr(ccc_filter)
    sFhc_syst = dblarr(ccc_filter)
    for i = 0,ccc_filter-1 do begin ; loop over all sources to which the method is applied
       ; Get statistical errors on Ftc and Fhc using Table 4 (me5, flux_ratio5, model5, band5)
       ; Hard band
       ind_subset = where((model5 eq bmf[i]) and (band5 eq 'hard'),ccc_subset)
       me5_subset = me5[ind_subset] & flux_ratio5_subset = flux_ratio5[ind_subset]
       MedEt_lo = MedEt[i] - sMedEt[i]
       MedEt_hi = MedEt[i] + sMedEt[i] 
       ind_close_lo = where(abs(MedEt_lo - me5_subset) eq min(abs(MedEt_lo - me5_subset)),ccc_close_lo) & ind_close_lo0 = ind_close_lo[0]
       ind_close_hi = where(abs(MedEt_hi - me5_subset) eq min(abs(MedEt_hi - me5_subset)),ccc_close_hi) & ind_close_hi0 = ind_close_hi[0]
       temp_sflux_ratio = abs(10.^flux_ratio5_subset[ind_close_hi0]-10.^flux_ratio5_subset[ind_close_lo0])/2.
       ; Y = X1 * X2, where X1 and X2 are independent variables. Error propagation is sY^2 = X2^2 * sX1^2 + X1^2 * sX2^2
       sFhc_stat[i] = sqrt(Fh[i]^2 * temp_sflux_ratio^2 + (Fhc[i]/Fh[i])^2 * sFh[i]^2)
       ; Full band
       ind_subset = where((model5 eq bmf[i]) and (band5 eq 'full'),ccc_subset)
       me5_subset = me5[ind_subset] & flux_ratio5_subset = flux_ratio5[ind_subset]
       MedEt_lo = MedEt[i] - sMedEt[i]
       MedEt_hi = MedEt[i] + sMedEt[i] 
       ind_close_lo = where(abs(MedEt_lo - me5_subset) eq min(abs(MedEt_lo - me5_subset)),ccc_close_lo) & ind_close_lo0 = ind_close_lo[0]
       ind_close_hi = where(abs(MedEt_hi - me5_subset) eq min(abs(MedEt_hi - me5_subset)),ccc_close_hi) & ind_close_hi0 = ind_close_hi[0]
       temp_sflux_ratio = abs(10.^flux_ratio5_subset[ind_close_hi0]-10.^flux_ratio5_subset[ind_close_lo0])/2.
       ; Y = X1 * X2, where X1 and X2 are independent variables. Error propagation is sY^2 = X2^2 * sX1^2 + X1^2 * sX2^2
       sFtc_stat[i] = sqrt(Ft[i]^2 * temp_sflux_ratio^2 + (Ftc[i]/Ft[i])^2 * sFt[i]^2)

       
      ; Get systematic errors on Ftc and Fhc using Table 4 (me5, flux_ratio5, model5, band5). These systematic errors include only uncertainty
      ; on knowledge of X-ray spectral models. They do not included other types of systematic errors, such as uncertainty on distance, or X-ray 
      ; variability.
      ; Get the "second nearest neighbour" model curves (call them left_model and right_model). 
      ; In the cases where bmf =[27, 27.5, 31.5, 32], we will use one-sided errors, or rather maximum value among low and hi systematic errors. 
      if (bmf[i] gt 27.5) then begin
         left_model = bmf[i] - 1
      endif else begin
         left_model = 27.0
      endelse

      if (bmf[i] lt 31.5) then begin
         right_model = bmf[i] + 1 
      endif else begin
         right_model = 32.0
      endelse
      
      ; Hard band
      ind_subset_left = where((model5 eq left_model) and (band5 eq 'hard'),ccc_subset_left)
      me5_subset_left = me5[ind_subset_left] & flux_ratio5_subset_left = flux_ratio5[ind_subset_left]
      ind_close_left = where(abs(MedEt[i] - me5_subset_left) eq min(abs(MedEt[i] - me5_subset_left)),ccc_close_left) & ind_close_left0 = ind_close_left[0]

      ind_subset_right = where((model5 eq right_model) and (band5 eq 'hard'),ccc_subset_right)
      me5_subset_right = me5[ind_subset_right] & flux_ratio5_subset_right = flux_ratio5[ind_subset_right]
      ind_close_right = where(abs(MedEt[i] - me5_subset_right) eq min(abs(MedEt[i] - me5_subset_right)),ccc_close_right) & ind_close_right0 = ind_close_right[0]

      syst_arr = [abs(Fhc[i]/Fh[i] - 10.^flux_ratio5_subset_left[ind_close_left0]), abs(Fhc[i]/Fh[i] - 10.^flux_ratio5_subset_right[ind_close_right0])]
      temp_sflux_ratio = max(syst_arr)
      ; Y = a * X, where "a" is a constant and X is an independent variable. Error propagation is sY^2 = a^2 * sX^2
      ; Here we have Fhc = (Fhc/Fh) * Fh, with Fh as a constant as a systematic uncertainty on Fh due to an uncertainty on knowledge of
      ; X-ray spectral models is 0.
      sFhc_syst[i] = sqrt(Fh[i]^2 * temp_sflux_ratio^2)
 
      ; Full band
      ind_subset_left = where((model5 eq left_model) and (band5 eq 'full'),ccc_subset_left)
      me5_subset_left = me5[ind_subset_left] & flux_ratio5_subset_left = flux_ratio5[ind_subset_left]
      ind_close_left = where(abs(MedEt[i] - me5_subset_left) eq min(abs(MedEt[i] - me5_subset_left)),ccc_close_left) & ind_close_left0 = ind_close_left[0]

      ind_subset_right = where((model5 eq right_model) and (band5 eq 'full'),ccc_subset_right)
      me5_subset_right = me5[ind_subset_right] & flux_ratio5_subset_right = flux_ratio5[ind_subset_right]
      ind_close_right = where(abs(MedEt[i] - me5_subset_right) eq min(abs(MedEt[i] - me5_subset_right)),ccc_close_right) & ind_close_right0 = ind_close_right[0]

      syst_arr = [abs(Ftc[i]/Ft[i] - 10.^flux_ratio5_subset_left[ind_close_left0]), abs(Ftc[i]/Ft[i] - 10.^flux_ratio5_subset_right[ind_close_right0])]
      temp_sflux_ratio = max(syst_arr)
      ; Y = a * X, where "a" is a constant and X is an independent variable. Error propagation is sY^2 = a^2 * sX^2
      ; Here we have Ftc = (Ftc/Ft) * Ft, with Ft as a constant as a systematic uncertainty on Ft due to an uncertainty on knowledge of
      ; X-ray spectral models is 0.
      sFtc_syst[i] = sqrt(Ft[i]^2 * temp_sflux_ratio^2)
    endfor ; end loop over all sources to which the method is applied
   


  ;; STEP 7. Save the derived by our photometry method quantities: Fh, Fhc, Ft, Ftc, logNH, and their errors + AE quantities: MedEh, MedEt, NCt, NCh + derived here errors on MedE
    print, "STEP 7. Saving results in xphot_out.sav file."  
    Fh_out = dblarr(ccc_all)  ; apparent hard-band flux in ergs/cm2/s ; linear version
    Fhc_out = dblarr(ccc_all) ; intrinsic (corrected for absorption) hard-band flux in ergs/cm2/s; linear version 
    Ft_out = dblarr(ccc_all)  ; apparent full-band flux in ergs/cm2/s ; linear version
    Ftc_out = dblarr(ccc_all) ; intrinsic full-band flux in ergs/cm2/s ; linear version
    logNH_out = dblarr(ccc_all) ; X-ray absorbing column density in cm-2 ; Log version
    sMedEt_out = dblarr(ccc_all)  ; statistical errors on full-band median energy, MedEt, in keV
    sMedEh_out = dblarr(ccc_all)  ; statistical errors on hard-band median energy, MedEh, in keV    
    sFh_out = dblarr(ccc_all) ; statistical errors on apparent flux in hard-band
    sFt_out = dblarr(ccc_all) ; statistical errors on apparent flux in full-band
    slogNH_stat_out = dblarr(ccc_all) ; statistical errors on LogNH
    slogNH_syst_out = dblarr(ccc_all) ; systematic errors on LogNH
    sFhc_stat_out = dblarr(ccc_all) ; statistical errors on had-band intrinsic flux
    sFtc_stat_out = dblarr(ccc_all) ; statistical errors on full-band intrinsic flux
    sFhc_syst_out = dblarr(ccc_all) ; systematic errors on had-band intrinsic flux
    sFtc_syst_out = dblarr(ccc_all) ; systematic errors on full-band intrinsic flux

    Fh_out[ind_filter] = Fh
    Fhc_out[ind_filter] = Fhc
    Ft_out[ind_filter] = Ft
    Ftc_out[ind_filter] = Ftc
    logNH_out[ind_filter] = logNH
    sMedEt_out[ind_filter] = sMedEt
    sMedEh_out[ind_filter] = sMedEh
    sFh_out[ind_filter] = sFh
    sFt_out[ind_filter] = sFt
    slogNH_stat_out[ind_filter] = slogNH_stat 
    slogNH_syst_out[ind_filter] = slogNH_syst
    sFhc_stat_out[ind_filter] = sFhc_stat
    sFtc_stat_out[ind_filter] = sFtc_stat
    sFhc_syst_out[ind_filter] = sFhc_syst
    sFtc_syst_out[ind_filter] = sFtc_syst

    Fh_out[ind_left] = -99.0
    Fhc_out[ind_left] = -99.0
    Ft_out[ind_left] = -99.0
    Ftc_out[ind_left] = -99.0
    logNH_out[ind_left] = -99.0
    sFh_out[ind_left] = -99.0
    sFt_out[ind_left] = -99.0
    slogNH_stat_out[ind_left] = -99.0
    slogNH_syst_out[ind_left] = -99.0
    sMedEt_out[ind_left] = -99.0
    sMedEh_out[ind_left] = -99.0
    sFhc_stat_out[ind_left] = -99.0
    sFtc_stat_out[ind_left] = -99.0
    sFhc_syst_out[ind_left] = -99.0
    sFtc_syst_out[ind_left] = -99.0 
    

    save, file='xphot_out.sav', NCt_out, NCh_out, MedEh_out, MedEt_out, fh_out, fhc_out, ft_out, ftc_out, lognh_out, sMedEt_out, sMedEh_out, sFh_out, sFt_out, slogNH_stat_out, slogNH_syst_out, sFhc_stat_out, sFtc_stat_out, sFhc_syst_out, sFtc_syst_out

end
